unit BarCode;

interface

uses
  SysUtils, Classes, Controls,Graphics;

type
  B09 = array[0..9] of string;
  Besq = array[0..1,0..9] of string;
const
  BinDig:B09 = ('000000','001011','001101','001110','010011','011001',
  '011100','010101','010110','011010');
  BinEsq:Besq = (('0001101','0011001','0010011','0111101','0100011','0110001',
  '0101111','0111011','0110111','0001011'),('0100111','0110011','0011011',
  '0100001','0011101','0111001','0000101','0010001','0001001','0010111'));
  BinDir:B09 = ('1110010','1100110','1101100','1000010','1011100','1001110',
  '1010000','1000100','1001000','1110100');

type
  TBarCode = class(TGraphicControl)
  private
    { Private declarations }
    FCode:string;
    procedure SetCode(eCode:string);
  protected
    { Protected declarations }
    procedure Paint;override;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent);override;
    function Bitmap:TBitmap;
    function CodeToBmp(eCode:string):TBitmap;
    function BinToBmp(eBin:string):TBitmap;
    function CodeToBin(eCode:string):string;
    function BinToCode(eBin:string):string;
    destructor Destroy;override;
  published
    { Published declarations }
    property Code:string read FCode write SetCode;
  end;

procedure Register;

implementation

//--------------------- registra o componente na paleta ----------------------//
procedure Register;
begin
  RegisterComponents('Bar Code', [TBarCode]);
end;

//-------------------------- quando cria o componente ------------------------//
constructor TBarCode.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  Width := 104;
  Height := 46;
  FCode := '7890123456789';
end;

//-------------------------- destroy o componente ----------------------------//
destructor TBarCode.Destroy;
begin
 inherited Destroy;
end;

//-------------- funo para pegar a imagem do cdigo de barra ---------------//
function TBarCode.Bitmap:TBitmap;
begin
 result := CodeToBmp(FCode);
end;

//-------------------------- desenha o componente ----------------------------//
procedure TBarCode.Paint;
begin
  Canvas.Draw(0,0,CodeToBmp(FCode));
end;
//-------------------------- Ajusta o code -----------------------------------//
procedure TBarCode.SetCode(eCode:string);
begin
  if (Length(eCode) = 13) then
  begin
    FCode := eCode;
    Repaint;
  end;
end;

//------------- transforma o cdigo em cdigo de barras ----------------------//
function TBarCode.CodeToBmp(eCode:string):TBitmap;
var
  eBin:string;
  i,ColWidth:integer;
  bmp:TBitmap;
begin
  bmp:=TBitmap.Create;
  eBin := CodeToBin(eCode);
  bmp.Width := 104;
  bmp.Height := 46;
  for i:= 1 to 95 do
  begin
    if (i in[1..3,46..50,93..95]) then ColWidth := 43 else ColWidth := 35;
    bmp.Canvas.MoveTo(i+7,1);
    if (eBin[i] = '1') then bmp.Canvas.LineTo(i+7,ColWidth);
  end;
  bmp.Canvas.Font.Name := 'Raavi';
  bmp.Canvas.TextOut(0,35,eCode[1]);
  bmp.Canvas.TextOut(11,35,Copy(eCode,2,6));
  bmp.Canvas.TextOut(57,35,Copy(eCode,8,6));
  result := bmp;
end;

//------------- transforma binario em cdigo de barras -----------------------//
function TBarCode.BinToBmp(eBin:string):TBitmap;
var
  eCode:string;
begin
  eCode := BinToCode(eBin);
  result := CodeToBmp(eCode);
end;

//-------------------- transforma cdigo em binario --------------------------//
function TBarCode.CodeToBin(eCode:string): string;
var
  i:integer;
  eBin:string;
begin
  eBin := '101';
  for i:= 1 to 6 do
    eBin := eBin + BinEsq[StrToInt(BinDig[StrToInt(eCode[1])][i]),
    StrToInt(eCode[i+1])];
  eBin := eBin + '01010';
  for i:= 8 to 13 do
    eBin := eBin + BinDir[StrToInt(eCode[i])];
  eBin := eBin + '101';
  result := eBin;
end;

//-------------------- transforma binrio em cdigo --------------------------//
function TBarCode.BinToCode(eBin:string): string;
var
  i,j,k:integer;
  Dig,CodEsq,CodDir:string;
  BinE,BinD:array[1..6] of string;
begin
  for i:= 1 to 6 do
  begin
    BinE[i] := Copy(eBin,(i*7)-3,7);
    BinD[i] := Copy(eBin,(i*7)+44,7);
    for j:= 0 to 9 do
    begin
      if (BinD[i] = BinDir[j]) then CodDir := CodDir + IntToStr(j);
      for k:= 0 to 1 do
        if (BinE[i] = BinEsq[k,j]) then
        begin
          CodEsq := CodEsq + IntToStr(j);
          Dig := Dig + IntToStr(k);
        end;
    end;
  end;
  for i:= 0 to 9 do if (BinDig[i] = Dig) then Dig := IntToStr(i);
  result := Dig + CodEsq + CodDir;
end;

end.
